"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const ajv = require("ajv");
const types_1 = require("sonarwhal/dist/src/lib/types");
const misc_1 = require("sonarwhal/dist/src/lib/utils/misc");
class BabelConfigParser extends types_1.Parser {
    constructor(sonarwhal) {
        super(sonarwhal);
        this.configFound = false;
        this.schema = misc_1.loadJSONFile(path.join(__dirname, 'schema', 'babelConfigSchema.json'));
        this.validator = new ajv({
            $data: true,
            allErrors: true,
            schemaId: 'id',
            verbose: true
        });
        this.validator.addMetaSchema(require('ajv/lib/refs/json-schema-draft-04.json'));
        sonarwhal.on('fetch::end::json', this.parseBabelConfig.bind(this));
        sonarwhal.on('scan::end', this.parseEnd.bind(this));
    }
    async parseEnd(scanEnd) {
        if (!this.configFound) {
            await this.sonarwhal.emitAsync('notfound::babel-config', scanEnd);
        }
    }
    async validateSchema(config, resource) {
        const validate = this.validator.compile(this.schema);
        const valid = validate(config);
        if (!valid) {
            const event = {
                errors: validate.errors,
                resource
            };
            await this.sonarwhal.emitAsync('invalid-schema::babel-config', event);
        }
        return valid;
    }
    async parseBabelConfig(fetchEnd) {
        const resource = fetchEnd.resource;
        const resourceFileName = path.basename(resource);
        const isPackageJson = resourceFileName === 'package.json';
        const isBabelrc = resourceFileName === '.babelrc';
        if (!isBabelrc && !isPackageJson) {
            return;
        }
        let config;
        try {
            const response = fetchEnd.response;
            const content = JSON.parse(response.body.content);
            if (isPackageJson && !content.babel) {
                return;
            }
            this.configFound = true;
            config = isPackageJson ? content.babel : content;
            const valid = await this.validateSchema(config, resource);
            if (!valid) {
                return;
            }
            const event = {
                config,
                resource
            };
            await this.sonarwhal.emitAsync('parse::babel-config', event);
        }
        catch (err) {
            const errorEvent = {
                error: err,
                resource
            };
            await this.sonarwhal.emitAsync('invalid-json::babel-config', errorEvent);
        }
    }
}
exports.default = BabelConfigParser;
